#!/bin/csh -f
#
# Script to compile & exercise pma_sam utility.
# Copyright (C) 2022  Terence Kelly
# Contact:  tpkelly @ { acm.org, cs.princeton.edu, eecs.umich.edu }
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

set b = pma_sam
set h = heap.pma
set printexitvalue

alias clean 'rm -f $b $h $h.cp'

# envar below helps when pm-gawk is not installed on $PATH
if (! $?PMGAWK ) then
    alias pm-gawk    'gawk --lint'
else
    alias pm-gawk "$PMGAWK --lint"
endif

echo "===================================================== clean up"
clean
if ( "clean" == $1 ) then
    exit
endif
echo "============================================== compile $b"
if ( $?PMAALLWARNINGS ) then
    set warningflags = `gawk '{print $1}' gcc_warnings_final`
else
    echo '    \!\!\! using bare minimum warning flags \!\!\!'
    set warningflags = '-Wall -Wextra'
endif
# note: assertions should not be disabled when compiling pma here
gcc -std=c99 $warningflags -O3    -o $b $b.c pma.c # max warnings
gcc -std=c99 $warningflags -Og -g -o $b $b.c pma.c # best debugging
echo "============================================= create heap file"
truncate -s 40960000 $h
setenv GAWK_PERSIST_FILE $h
echo "============================================== ls -l heap file"
ls -l $h
echo "============================================== du -h heap file"
du -h $h
echo "================================= gawk hello & du -h heap file"
set i = 1
while ($i < 2)  # set upper bound higher to measure memory leaks
    echo -n $i ' '
    pm-gawk 'BEGIN{printf "hello  "}'
    du -h $h
    @ i ++
end
echo "================================== fill gawk persistent arrays"
pm-gawk 'BEGIN{a[1] = "x"; for(i=0;i<24;i++) a[1] = a[1] a[1]}'
echo -n 'size of large array entry: '
pm-gawk 'BEGIN{print a[1]}' | wc -c
pm-gawk 'BEGIN{for(b=1;b<=47;b++) c[b]=b}'
pm-gawk 'BEGIN{t=0; for(b=1;b<=47;b++) {t+=c[b];} print "small array sum: " t}'
echo "============================================== du -h heap file"
du -h $h
echo '(should be around 33M)'
echo "======================================== delete big gawk array"
pm-gawk 'BEGIN{delete a}'
echo "============================================== du -h heap file"
du -h $h
echo '(should still be around 33M)'
echo "==================================== call $b on heap file"
# over-write de-allocated memory in the persistent heap with zero
# bytes, enabling fallocate below to re-sparsify the heap file
./$b $h
echo "============================================== du -h heap file"
du -h $h
echo '(should still be around 33M)'
echo "================================================== cp --sparse"
# this should in principle create a sparse copy but on my
# fedora system it doesn't sparsify as expected; sheesh
cp --sparse=always $h $h.cp
echo "=================================================== du -h copy"
du -h $h.cp
echo '(copy should be much smaller, under 1M)'
echo '(but cp --sparse does not always work)'
echo "======================================= fallocate -d heap file"
# this should "dig holes" in the heap file, i.e., sparsify it
# in-place by de-allocating storage resources beneath pages
# containing only zero bytes
fallocate -d $h
echo "============================================== du -h heap file"
du -h $h
echo '(should be much smaller, under 1M)'
echo "============================== check heap file for small array"
# confirm that persistent heap is still usable by gawk; note that
# it's important to re-set t=0 below; pm-gawk variables are persistent
pm-gawk 'BEGIN{t=0; for(b=1;b<=47;b++) {t+=c[b];} print "small array sum: " t}'
pm-gawk 'BEGIN{print "Euler formula:   " 47*48/2}'
echo '(two numbers above should be identical)'
echo "===================================================== clean up"
clean
echo "========================================================= done"

